import React, { useState } from 'react'
import Button from '@material-ui/core/Button'
import TextField from '@material-ui/core/TextField'
import Dialog from '@material-ui/core/Dialog'
import DialogActions from '@material-ui/core/DialogActions'
import DialogContent from '@material-ui/core/DialogContent'
import DialogContentText from '@material-ui/core/DialogContentText'
import DialogTitle from '@material-ui/core/DialogTitle'
import { useRouter } from 'next/router'
import {
  checkPortfolioExists,
  checkUserValidty,
  createPortfolio
} from '../../../actions/portfolio'

export default function PortfolioGenerateDialog(props) {
  const router = useRouter()
  const [username, setUsername] = useState('')
  const [stofUsername, setStofUsername] = useState('')
  const [mediumUsername, setMediumUsername] = useState('')

  const handleChange = (e) => {
    console.log(e.target.value)
    setUsername(e.target.value.toLowerCase())
  }
  const handleStofChange = (e) => {
    console.log(e.target.value)
    setStofUsername(e.target.value.toLowerCase())
  }
  const handleMediumChange = (e) => {
    console.log(e.target.value)
    setMediumUsername(e.target.value.toLowerCase())
  }
  const navigateToPortfolio = (username) => {
    console.log('username::', username)
    router.push(`/portfolio/${username}`)
  }
  const createUserPortfolio = async (git_username,stof_username,medium_username) => {
    const portfolio = await createPortfolio(git_username,stof_username,medium_username)
    return portfolio && portfolio.valid ? true : false
  }

  const portfolioGeneration = async (git_username,stof_username,medium_username) => {
    try {
      const profileValidity = await checkGithubProfile(git_username)
      if (profileValidity) {
        const portfolioValidity = await checkPortfolioAvailability(git_username)
        if (portfolioValidity) {
          navigateToPortfolio(git_username)
        } else {
          const portfolio = await createUserPortfolio(git_username,stof_username,medium_username)
          if (portfolio) {
            navigateToPortfolio(git_username)
          } else {
            console.log('something went wrong, show it using toast')
          }
        }
      } else {
        console.log('username is not exists on github')
      }
    } catch (err) {
      console.log('something went wrong,', err)
    }
  }

  // check user portfolio availabilty from the backend
  const checkPortfolioAvailability = async (username) => {
    const response = await checkPortfolioExists(username)

    return response && response.valid ? true : false
  }

  const checkGithubProfile = async (username) => {
    const validity = await checkUserValidty(username)
    console.log('git account,', validity)
    return validity.user.valid ? true : false
  }

  return (
    <div>
      <Dialog
        open={props.open}
        onClose={props.handleClose}
        aria-labelledby="form-dialog-title"
      >
        <DialogTitle id="form-dialog-title">Generate Portfolio</DialogTitle>
        <DialogContent>
          <DialogContentText>
            Please enter user's github username to generate the dynamic
            portfolio.
          </DialogContentText>
          <TextField
            value={username}
            onChange={handleChange}
            autoFocus
            margin="dense"
            id="username"
            label="GitHub Username"
            type="text"
            fullWidth
          />
          <TextField
            value={stofUsername}
            onChange={handleStofChange}
            autoFocus
            margin="dense"
            id="stof_username"
            label="Stackoverflow Username"
            type="text"
            fullWidth
          />
          <TextField
            value={mediumUsername}
            onChange={handleMediumChange}
            autoFocus
            margin="dense"
            id="medium_username"
            label="medium Username"
            type="text"
            fullWidth
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={props.handleClose} color="primary">
            Cancel
          </Button>
          <Button onClick={() => portfolioGeneration(username,stofUsername,mediumUsername)} color="primary">
            Generate
          </Button>
        </DialogActions>
      </Dialog>
    </div>
  )
}
