import {
  Avatar,
  Box,
  Card,
  CardMedia,
  Chip,
  Container,
  Grid,
  makeStyles,
  Paper,
  Typography
} from '@material-ui/core'
import { Rating } from '@material-ui/lab'
import { useRouter } from 'next/router'
import React, { useEffect, useState } from 'react'
import {
  chooseClass,
  getAllPortfolios,
  getGithubInfo,
  saveGithubInfo
} from '../../actions/portfolio'
import Header from '../../components/layout/header'
import BlogCard from '../../components/portfolio/blog-card/blog-card'
import FavLanguageCard from '../../components/portfolio/fav-language-card/fav-language-card'
import GitSummaryCard from '../../components/portfolio/git-summary-card/git-summary-card'
import ReputationIndicator from '../../components/portfolio/reputation-indicator'
import Storyline from '../../components/portfolio/story-line'
import TimeLanguageCard from '../../components/portfolio/time-language-card/time-language-card'
import { Chrono } from 'react-chrono'
import StofTopPostCard from '../../components/portfolio/top-stof-post-card'
import AppBar from '@material-ui/core/AppBar'
import Tabs from '@material-ui/core/Tabs'
import Tab from '@material-ui/core/Tab'
import PhoneIcon from '@material-ui/icons/Phone'
import FavoriteIcon from '@material-ui/icons/Favorite'
import PersonPinIcon from '@material-ui/icons/PersonPin'
import HelpIcon from '@material-ui/icons/Help'
import ShoppingBasket from '@material-ui/icons/ShoppingBasket'
import ThumbDown from '@material-ui/icons/ThumbDown'
import moment from 'moment'

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
    marginTop: '85px'
  },
  tabSection: {
    flexGrow: 1,
    width: '100%',
    backgroundColor: theme.palette.background.paper
  },
  repoChips: {
    display: 'flex',
    justifyContent: 'center',
    flexWrap: 'wrap',
    '& > *': {
      margin: theme.spacing(0.5)
    }
  },
  paper2: {
    padding: '15px',
    textAlign: 'center',
    color: theme.palette.text.secondary
  },
  propic: {
    height: '200px'
  },
  favLang: {
    height: '100px'
  },
  cardWidth: {
    maxWidth: 345
  },
  chipMl: {
    marginRight: '15px'
  },
  para: {
    fontSize: '18px'
  }
}))

function Porto({ gitInfo }) {
  const classes = useStyles();
  const router=useRouter();
  /* const router=useRouter();
    const {username}=router.query; */

  // Tabs for Repos
  const [value, setValue] = useState(1)

  const handleChange = (event, newValue) => {
    console.log('eve', newValue)
    setValue(newValue)
  }
  const navigateToRepo=(repoName)=>{
    window.open(`https://github.com/${repoName}`, '_blank').focus();
  }
  const navigateToStofPost=(postUrl)=>{
    window.open(`https://stackoverflow.com${postUrl}`, '_blank').focus();
  }
  const navigateToMediumBlog=(blogUrl)=>{
    window.open(`https://medium.com${blogUrl}`, '_blank').focus();
  }
  function a11yProps(index) {
    return {
      id: `scrollable-force-tab-${index}`,
      'aria-controls': `scrollable-force-tabpanel-${index}`
    }
  }
  function TabPanel(props) {
    const { children, value, index, ...other } = props

    return (
      <div
        role="tabpanel"
        hidden={value !== index}
        id={`scrollable-force-tabpanel-${index}`}
        aria-labelledby={`scrollable-force-tab-${index}`}
        {...other}
      >
        {value === index && (
          <Box p={3}>
            <Typography>{children}</Typography>
          </Box>
        )}
      </div>
    )
  }
  const [overallScore, setOverallScore] = useState('')
  const [overallClass, setOverallClass] = useState('')
  console.log('from front end', gitInfo)
  const gitStats = {
    location: gitInfo.location,
    company: gitInfo.company,
    totalPRs: gitInfo.total_prs,
    totalStars: gitInfo.total_stars,
    contributedTo: gitInfo.contributed_to,
    totalIssues: gitInfo.total_issues
  }
  const timeLineItems = []
  gitInfo.stof_top_posts.map((i) => timeLineItems.push({ title: i.date }))
  // add github join date
  timeLineItems.push({ title:moment(gitInfo.created_at).format("MMM DD 'YY") })
  // add stackOverflow date
  gitInfo.stof_connection && timeLineItems.push({ title: "Jun 24 '13"  })
  // calculate overall score
  const calculateOverallScore = () => {
    let score = gitInfo.stof_rank.score * 0.2 + gitInfo.coding.level * 0.8
    setOverallScore(score)
    let selectedClass = chooseClass(score)
    setOverallClass(selectedClass)
  }

  useEffect(() => {
    console.log('Finishes loading.....')
    const saveGitInfo = async () => {
      console.log('start.....')
      await saveGithubInfo(gitInfo)
      console.log('end.....')
    }
    saveGitInfo()
    gitInfo.stof_connection && calculateOverallScore()
  }, [])

  return (
    <div className={classes.root}>
      <Header></Header>
      <Container>
        <Grid container spacing={8}>
          {/* ProPic and Indicators */}
          <Grid item container xs={12} sm={6} spacing={3}>
            <Grid item xs={12} sm={6}>
              <Paper className={classes.paper2}>
                <Card className={classes.cardWidth}>
                  <CardMedia
                    className={classes.propic}
                    image={gitInfo.avatar_url}
                    title="Profile picture"
                  />
                </Card>
              </Paper>
              <Typography variant="h4">{gitInfo.name}</Typography>
            </Grid>
            <Grid
              item
              container
              xs={12}
              sm={6}
              direction="column"
              justify="space-around"
            >
              {/* Reputation Bars */}
              <ReputationIndicator
                area="Coding"
                rank={gitInfo.coding}
              ></ReputationIndicator>
              {gitInfo.stof_connection && (
                <ReputationIndicator
                  area="Q/A"
                  rank={{
                    level: gitInfo.stof_rank.score,
                    badge: gitInfo.stof_rank.class
                  }}
                ></ReputationIndicator>
              )}
              {gitInfo.stof_connection && (
                <ReputationIndicator
                  area="Overall"
                  rank={{ level: overallScore, badge: overallClass }}
                ></ReputationIndicator>
              )}

              {/* Reputation Bars End */}
            </Grid>
          </Grid>
          {/*End ProPic and Indicators */}
          <Grid item container xs={12} sm={6} spacing={3}>
            {/* Fav Language 2 Blocks */}
            <div className={classes.tabSection}>
              <AppBar position="static" color="default">
                <Tabs
                  value={value}
                  onChange={handleChange}
                  variant="scrollable"
                  scrollButtons="on"
                  indicatorColor="primary"
                  textColor="primary"
                  aria-label="scrollable force tabs example"
                >
                  {gitInfo.contribute_high_repos &&
                  gitInfo.contribute_high_repos.length > 0 ? (
                    <Tab
                      label="Opensource High"
                      icon={<PhoneIcon />}
                      {...a11yProps(0)}
                    />
                  ) : (
                    <div></div>
                  )}
                  {gitInfo.contribute_mid_repos &&
                  gitInfo.contribute_mid_repos.length > 0 ? (
                    <Tab
                      label="Opensource Mid"
                      icon={<FavoriteIcon />}
                      {...a11yProps(1)}
                    />
                  ) : (
                    <div></div>
                  )}
                  {gitInfo.contribute_low_repos &&
                  gitInfo.contribute_low_repos.length > 0 ? (
                    <Tab
                      label="Opensource Low"
                      icon={<PersonPinIcon />}
                      {...a11yProps(2)}
                    />
                  ) : (
                    <div></div>
                  )}
                  {gitInfo.individual_high_repos &&
                  gitInfo.individual_high_repos.length > 0 ? (
                    <Tab
                      label="Individual High"
                      icon={<HelpIcon />}
                      {...a11yProps(3)}
                    />
                  ) : (
                    <div></div>
                  )}
                  {gitInfo.individual_mid_repos &&
                  gitInfo.individual_mid_repos.length > 0 ? (
                    <Tab
                      label="Individual Mid"
                      icon={<ShoppingBasket />}
                      {...a11yProps(4)}
                    />
                  ) : (
                    <div></div>
                  )}
                  {gitInfo.individual_low_repos &&
                  gitInfo.individual_low_repos.length > 0 ? (
                    <Tab
                      label="Individual Low"
                      icon={<ThumbDown />}
                      {...a11yProps(5)}
                    />
                  ) : (
                    <div></div>
                  )}
                </Tabs>
              </AppBar>
              {gitInfo.contribute_high_repos &&
                gitInfo.contribute_high_repos.length > 0 && (
                  <TabPanel
                    
                    value={value}
                    index={0}
                  >
                    <div className={classes.repoChips}>
                      {gitInfo.contribute_high_repos.map((rep, id) => (
                        <Chip
                          key={id}
                          variant="outlined"
                          label={rep.substring(rep.indexOf('/') + 1)}
                          onClick={()=>navigateToRepo(rep)}
                        />
                      ))}
                    </div>
                  </TabPanel>
                )}
              {gitInfo.contribute_mid_repos &&
                gitInfo.contribute_mid_repos.length > 0 && (
                  <TabPanel
                    
                    value={value}
                    index={1}
                  >
                    <div className={classes.repoChips}>
                      {gitInfo.contribute_mid_repos.map((rep, id) => (
                        <Chip
                          key={id}
                          variant="outlined"
                          label={rep.substring(rep.indexOf('/') + 1)}
                          onClick={()=>navigateToRepo(rep)}
                        />
                      ))}
                    </div>
                  </TabPanel>
                )}
              {gitInfo.contribute_low_repos &&
                gitInfo.contribute_low_repos.length > 0 && (
                  <TabPanel
                    
                    value={value}
                    index={2}
                  >
                    <div className={classes.repoChips}>
                      {gitInfo.contribute_low_repos.map((rep, id) => (
                        <Chip
                          key={id}
                          variant="outlined"
                          label={rep.substring(rep.indexOf('/') + 1)}
                          onClick={()=>navigateToRepo(rep)}
                        />
                      ))}
                    </div>
                  </TabPanel>
                )}
              {gitInfo.individual_high_repos &&
                gitInfo.individual_high_repos.length > 0 && (
                  <TabPanel
                    
                    value={value}
                    index={3}
                  >
                    <div className={classes.repoChips}>
                      {gitInfo.individual_high_repos.map((rep, id) => (
                        <Chip
                          key={id}
                          variant="outlined"
                          label={rep.substring(rep.indexOf('/') + 1)}
                          onClick={()=>navigateToRepo(rep)}
                        />
                      ))}
                    </div>
                  </TabPanel>
                )}
              {gitInfo.individual_mid_repos &&
                gitInfo.individual_mid_repos.length > 0 && (
                  <TabPanel
                    
                    value={value}
                    index={4}
                  >
                    <div className={classes.repoChips}>
                      {gitInfo.individual_mid_repos.map((rep, id) => (
                        <Chip
                          key={id}
                          variant="outlined"
                          label={rep.substring(rep.indexOf('/') + 1)}
                          onClick={()=>navigateToRepo(rep)}
                        />
                      ))}
                    </div>
                  </TabPanel>
                )}
              {gitInfo.individual_low_repos &&
                gitInfo.individual_low_repos.length > 0 && (
                  <TabPanel
                    
                    value={value}
                    index={5}
                  >
                    <div className={classes.repoChips}>
                      {gitInfo.individual_low_repos.map((rep, id) => (
                        <Chip
                          key={id}
                          variant="outlined"
                          label={rep.substring(rep.indexOf('/') + 1)}
                          onClick={()=>navigateToRepo(rep)}
                        />
                      ))}
                    </div>
                  </TabPanel>
                )}
            </div>

            {/* End Fav Language 2 Blocks */}
            {/* Chips (Languages) */}
            <Grid item container xs={12} spacing={1}>
              {/* <Chip
                className={classes.chipMl}
                label="JavaScript"
                avatar={<Avatar src="/images/portfolio/javaLogo.png" />}
                variant="outlined"
              />
              <Chip
                className={classes.chipMl}
                label="JavaScript"
                avatar={<Avatar src="/images/portfolio/javaLogo.png" />}
                variant="outlined"
              /> */}
              <Typography className={classes.para}>{gitInfo.bio}</Typography>
            </Grid>
            {/* End Chips (Languages) */}
          </Grid>
          {/* Github Stats */}
          <Grid container item xs={12}>
            <Grid item xs={4}>
              <FavLanguageCard languages={gitInfo.git_langs}></FavLanguageCard>
            </Grid>
            <Grid item xs={4}>
              <TimeLanguageCard></TimeLanguageCard>
            </Grid>
            <Grid item xs={4}>
              <GitSummaryCard info={gitStats}></GitSummaryCard>
            </Grid>
          </Grid>
          {/* End Github Stats */}
          {/* Story Line */}
          <Grid
            container
            item
            xs={12}
            justify="center"
            alignItems="flex-end"
            direction="row"
          >
            <Grid item xs={8}>
              {/* <Storyline userData={gitInfo.created_at}></Storyline> */}
              <Chrono
                mode="VERTICAL_ALTERNATING"
                scrollable={{ scrollbar: false }}
                items={timeLineItems}
                useReadMore={false}
                hideControls={true}
              >
                {gitInfo.stof_top_posts.map((post, id) => (
                  <StofTopPostCard
                    key={id}
                    name={post.name}
                    upvotes={post.upvotes}
                    navigate={()=>navigateToStofPost(post.url)}
                  ></StofTopPostCard>
                ))}
                <div>
                  <div
                    style={{
                      width: '250px',
                      height: '150px',
                      textAlign: 'center'
                    }}
                  >
                    <Typography variant="h4" component="h4">
                      Joined Github
                    </Typography>
                    <img
                      style={{ maxWidth: '60%', maxHeight: '60%',marginTop:'20px' }}
                      alt="test"
                      src="https://cdn.tutsplus.com/net/uploads/2013/08/github-collab-retina-preview.gif"
                    />
                  </div>
                </div>
                {gitInfo.stof_connection && (
                  <div>
                    <div
                      style={{
                        width: '250px',
                        height: '150px',
                        textAlign: 'center'
                      }}
                    >
                      <Typography variant="h4" component="h4">
                        Joined Stackoverflow
                      </Typography>
                      <img
                        style={{ maxWidth: '50%', maxHeight: '50%', marginTop:'20px' }}
                        alt="test"
                        src="/images/portfolio/stofMain.png"
                      />
                    </div>
                  </div>
                )}
              </Chrono>
            </Grid>
          </Grid>
          {/* End Story Line */}
          {gitInfo.medium_connection && gitInfo.medium_blogs.map((blog, id) => (
                        <Grid key={id} item xs={6} sm={3}>
                        <BlogCard title={blog.title}
                    desc={blog.desc}
                    image={blog.image}
                    navigate={()=>navigateToMediumBlog(blog.url)}></BlogCard>
                      </Grid>
                      ))}
          {/* <Grid item xs={6} sm={3}>
            <BlogCard></BlogCard>
          </Grid>
          <Grid item xs={6} sm={3}>
            <BlogCard></BlogCard>
          </Grid>
          <Grid item xs={6} sm={3}>
            <BlogCard></BlogCard>
          </Grid>
          <Grid item xs={6} sm={3}>
            <BlogCard></BlogCard>
          </Grid> */}

          {/* <Grid item xs={12}>
            <Paper className={classes.paper2}>xs=12</Paper>
          </Grid> */}
        </Grid>
      </Container>
    </div>
  )
}
export default Porto

export async function getStaticPaths() {
  // const posts = ['tmKamal', 'grayAlpha', 'greyAlpha','dasunx']
  const posts = await getAllPortfolios()

  // Get the paths we want to pre-render based on posts
  const paths = posts.map((post) => ({
    params: { username: post.username.toLowerCase() }
  }))

  // { fallback: false } means other routes should 404.
  return { paths, fallback: false }
}

export async function getStaticProps({ params }) {
  // get gitInfo data based on slug
  console.log('username cased:', params.username.toLowerCase())
  const gitInfo = await getGithubInfo(params.username.toLowerCase()) //response data [ single gitInfo] recived with array brackets
  return { props: { gitInfo } }
}
