import React, { useEffect,useState } from 'react'
import { Button, Container, Grid, Icon, Typography } from '@material-ui/core'
import { makeStyles } from '@material-ui/core'
import Header from '../../components/layout/header'
import LeaderBoardTable from '../../components/portfolio/leaderboard-table/leaderboard-table'
import PortfolioGenerateDialog from '../../components/portfolio/portfolio-generate-dialog/portfolio-generate-dialog'
import { fetchLeaderboard } from '../../actions/leaderboard'


const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
    marginTop: '85px'
  },
  paper2: {
    padding: '15px',
    textAlign: 'center',
    color: theme.palette.text.secondary
  },
  propic: {
    height: '200px'
  },
  favLang: {
    height: '100px'
  },
  cardWidth: {
    maxWidth: 345
  },
  chipMl: {
    marginRight: '15px'
  }
}))
export default function LeaderBoard() {
  
  const classes = useStyles()
  const [open, setOpen] = React.useState(false);
  const [leaderboardData, setLeaderboardData] = useState([]);

  const handleClickOpen = () => {
    setOpen(true);
  };

  const handleClose = () => {
    setOpen(false);
  };
  
  useEffect(() => {
    console.log('Finishes loading.....')
    const fetchLeaderData = async () => {
      const response=await fetchLeaderboard();
      console.log('res12',response);
      setLeaderboardData(response);
    }
    fetchLeaderData();
  }, [])
  

  return (
    <div className={classes.root}>
      <Header></Header>
      <Container>
        <Grid container justify="space-between">
          <Grid item>
            <Button
              variant="contained"
              color="primary"
              className={classes.button}
              endIcon={<Icon>send</Icon>}
              size="large"
            >
              To My Portfolio
            </Button>
          </Grid>
          <Grid item>
            <Button
              variant="outlined"
              color="primary"
              className={classes.button}
              endIcon={<Icon>send</Icon>}
              size="large"
              onClick={handleClickOpen}
            >
              Generate Portfolio
            </Button>
          </Grid>
        </Grid>

        <Typography variant="h3" component="h1" gutterBottom>
          LeaderBoard
        </Typography>
        <LeaderBoardTable data={leaderboardData}></LeaderBoardTable>
      </Container>
      <PortfolioGenerateDialog open={open} handleClose={handleClose} ></PortfolioGenerateDialog>
    </div>
  )
}
