const moment = require("moment");

function normalcdf(mean, sigma, to) {
  var z = (to - mean) / Math.sqrt(2 * sigma * sigma);
  var t = 1 / (1 + 0.3275911 * Math.abs(z));
  var a1 = 0.254829592;
  var a2 = -0.284496736;
  var a3 = 1.421413741;
  var a4 = -1.453152027;
  var a5 = 1.061405429;
  var erf =
    1 - ((((a5 * t + a4) * t + a3) * t + a2) * t + a1) * t * Math.exp(-z * z);
  var sign = 1;
  if (z < 0) {
    sign = -1;
  }
  return (1 / 2) * (1 + sign * erf);
}

function calculateRank({
  totalRepos,
  totalCommits,
  contributions,
  followers,
  totalPRs,
  issues,
  stargazers,
  createdAt
}) {
  const COMMITS_WEIGHT = 1.65;
  const CONTRIBS_WEIGHT = 1.65;
  const ISSUES_WEIGHT = 1;
  const STARS_WEIGHT = 0.75;
  const PRS_WEIGHT = 0.5;
  const FOLLOWERS_WEIGHT = 0.45;
  const REPO_WEIGHT = 1;

  const ALL_WEIGHTS =
    CONTRIBS_WEIGHT +
    ISSUES_WEIGHT +
    STARS_WEIGHT +
    PRS_WEIGHT +
    FOLLOWERS_WEIGHT +
    REPO_WEIGHT;

  const LVL_GURU = 1;
  const LVL_PROFESSIONAL = 25;
  const LVL_ADVANCED = 45;
  const LVL_INTERMEDIATE = 55;
  const LVL_BEGINNER = 80;
  const LVL_NEWBIE = 100

  const TOTAL_VALUES =
    LVL_GURU + LVL_ADVANCED + LVL_INTERMEDIATE + LVL_NEWBIE+5;

  const rewardPoints = (
    totalCommits * COMMITS_WEIGHT +
    contributions * CONTRIBS_WEIGHT +
    issues * ISSUES_WEIGHT +
    stargazers * STARS_WEIGHT +
    totalPRs * PRS_WEIGHT +
    followers * FOLLOWERS_WEIGHT + 
    totalRepos * REPO_WEIGHT 
  ) / 100;
  
  const todayDate=new moment(Date.now());
  const joinedDate=new moment(createdAt);
  const profileAge=moment.duration(todayDate.diff(joinedDate)).asDays();
  console.log('day duration',moment.duration(todayDate.diff(joinedDate)).asDays());
  
  let penaltyPoints=0;
  // penalty calculation
  if(totalCommits<1000){
    penaltyPoints= (totalCommits==0?(-1000):( -1000/totalCommits))+penaltyPoints; 
    console.log('Commit-Penalty:',penaltyPoints);
  }
  
  if(totalPRs<15){
    
    penaltyPoints=(totalPRs==0?(-150): (-1500/(totalPRs*100)))+penaltyPoints;
    console.log('PRs-Penalty:',penaltyPoints);
  }
  
  if(profileAge<365){
    penaltyPoints= (profileAge==0?(-365):( -365/profileAge))+penaltyPoints;
    console.log('Age-Penalty:',penaltyPoints); 
  }
  console.log('total-Rewards',rewardPoints);
  console.log('total-Penalty',penaltyPoints);
  
  const totalScore=rewardPoints+penaltyPoints;
  
  const normalizedScore = normalcdf(totalScore, TOTAL_VALUES, ALL_WEIGHTS) * 100;
  console.log('avg is:',totalScore);
    console.log('sigma is:',TOTAL_VALUES);
    console.log('to is:',ALL_WEIGHTS);
    console.log('===================');

  let level = 0;
  let rank='';

  if (normalizedScore < LVL_GURU) {
    level = 95;
    rank='Guru'
  }
  if (
    normalizedScore >= LVL_GURU &&
    normalizedScore < LVL_PROFESSIONAL
  ) {
    level = 80;
    rank='Professional'
  }
  if (
    normalizedScore >= LVL_PROFESSIONAL &&
    normalizedScore < LVL_ADVANCED
  ) {
    level = 70;
    rank='Advanced'
  }
  if (normalizedScore >= LVL_ADVANCED && normalizedScore < LVL_INTERMEDIATE) {
    level = 50;
    rank='Intermediate'
  }
  if (normalizedScore >= LVL_INTERMEDIATE && normalizedScore < LVL_BEGINNER) {
    level = 30;
    rank='Beginner'
  }
  if(normalizedScore>=LVL_BEGINNER && normalizedScore<LVL_NEWBIE){
    level=10;
    rank='Newbie'
  }

  return { level, score: normalizedScore, badge:rank, points:totalScore };
}

module.exports = calculateRank;
