const { default: axios } = require("axios");
const {
  getGithubInfo,
  checkGitUserAvailability,
} = require("../actions/shared-action");
const HttpError = require("../helpers/http-error");
const User = require("../models/user");
const { spawn } = require('child_process');
const path=require('path');

const checkUserAvailabity = async (req, res, next) => {
  const reqUsername = req.params.username;
  const username = reqUsername.toLowerCase();

  try {
    const user = await User.findOne({ username });
    console.log(username);
    if (user) {
      const gitInfo = await getGithubInfo(username);
      return res.status(200).json({
        msg: "User portfolio exists",
        info: gitInfo,
        valid: true,
      });
    } else {
      return res.status(200).json({
        msg: "No portfolio exists",
        valid: false,
      });
    }
  } catch (err) {
    return res.status(304).json({
      msg: "something else went wrong:",
      err,
    });
  }
};

const getAllUserNames = async (req, res, next) => {
  let usernames;
  try {
    usernames = await User.find().select("username");
    res.status(201).json(usernames);
  } catch (err) {
    res.status(302).json({ msg: "something went wrong::", err });
  }
};
const sortedLeaderboardUsers = async (req, res, next) => {
  let usernames;
  try {
    usernames = await User.find()
      .select("username name coding")
      .sort({ "coding.points": -1 });

    res.status(201).json(usernames);
  } catch (err) {
    res.status(302).json({ msg: "something went wrong::", err });
  }
};

const createUser = async (req, res, next) => {
  const reqUsername = req.params.username;
  const username = reqUsername.toLowerCase();
  let user;
  try {
    user = await User.findOne({ username });
    if (user) {
      console.log("user already exists");
      return res.status(201).json({ msg: "user exists", valid: false });
    } else {
      // call github graphql api
      const gitInfo = await getGithubInfo(username);

   
      let newUser = new User();
      // cal stof scrapper
      // call stackoverflow web scrapper
      const pythonScript = spawn("python", [
        path.join(__dirname, "..", "python", "portfolio", "main.py"),newUser._id,'351048'
      ]);
      pythonScript.stdout.on("data", (data) => {
        console.log("Python test output", data.toString());
      });

      pythonScript.stdout.on("end", () => {
        console.log("Python script has finshed executing");
      });

      
      newUser.username = username.toLowerCase();
      newUser.name = gitInfo.stats.name;
      newUser.avatar_url = gitInfo.userBio.avatar_url;
      newUser.bio = gitInfo.userBio.bio;
      newUser.company = gitInfo.userBio.company;
      newUser.location = gitInfo.userBio.location;
      newUser.created_at = gitInfo.userBio.created_at;
      newUser.coding.level = gitInfo.stats.rank.level;
      newUser.coding.score = gitInfo.stats.rank.score;
      newUser.coding.badge = gitInfo.stats.rank.badge;
      newUser.coding.points = gitInfo.stats.rank.points;
      newUser.total_prs = gitInfo.stats.totalPRs;
      newUser.total_issues = gitInfo.stats.totalIssues;
      newUser.total_stars = gitInfo.stats.totalStars;
      newUser.total_commits = gitInfo.stats.totalCommits;
      newUser.contributed_to = gitInfo.stats.contributedTo;
      newUser.total_repos = gitInfo.stats.totalRepositories;
      newUser.total_followers = gitInfo.stats.followers;
      newUser.is_hireable = gitInfo.stats.isHireable;
      newUser.contribute_high_repos = gitInfo.stats.highRepos;
      newUser.contribute_mid_repos = gitInfo.stats.midRepos;
      newUser.contribute_low_repos = gitInfo.stats.lowRepos;
      newUser.individual_high_repos = gitInfo.stats.highIndividualRepos;
      newUser.individual_mid_repos = gitInfo.stats.midIndividualRepos;
      newUser.individual_low_repos = gitInfo.stats.lowIndividualRepos;
      newUser.profile_age = gitInfo.stats.profileAge;
      newUser.total_discussions = gitInfo.stats.totalDiscussions;
      newUser.answered_discussions = gitInfo.stats.answeredDiscussions;
      // languages
      if (gitInfo.toLang !== null) {
        Object.values(gitInfo.topLang).forEach((lang) => {
          newUser.git_langs.push({ name: lang.name, lines: lang.size });
        });
      }
      newUser.latest_update = Date.now();

      const userRes = await newUser.save();
      console.log("mongoResponse:", userRes);
      
      return res.status(201).json({
        msg: "user has been created successfully",
        response: userRes,
        valid: true,
      });
    }
  } catch (err) {
    console.log("error:", err);
    return res.status(302).json({
      msg: `something went wrong:${err}`,
      valid: false,
    });
  }
};

const getUserById = async (req, res, next) => {
  const { uname } = req.params;
  let selectedPortfolio;
  try {
    selectedPortfolio = await User.findOne({ username: uname });
    if (!selectedPortfolio) {
      return res.status(404).json({
        msg: `Oops! User not found`,
      });
    } else {
      return res.status(200).json({
        portfolio: selectedPortfolio,
      });
    }
  } catch (err) {
    console.log("error:", err);
    return res.status(302).json({
      msg: `Snap! Error: ${err}`,
    });
  }
};

// // check whether git account available or not via github vercel serverless api
const checkGitAccountAvailability = async (req, res, next) => {
  const username = req.params.username;
  try {
    const gitInfo = await checkGitUserAvailability(username);
    return res.status(200).json(gitInfo);
  } catch (err) {
    return res
      .status(406)
      .json({ msg: `something went wrong: ${err}`, valid: false });
  }
};

const saveUserGitInfo = async (req, res, next) => {
  const {
    username,
    name,
    avatar_url,
    bio,
    company,
    location,
    created_at,
    git_level,
    git_score,
    git_badge,
    git_points,
    total_prs,
    total_commits,
    total_issues,
    total_stars,
    contributed_to,
  } = req.body;
  console.log("retrived Data set", req.body);

  const lowercasedUser = username.toLowerCase();

  try {
    console.log("lowerCasedUser", lowercasedUser);
    let user = await User.findOne({ username: lowercasedUser });
    console.log("user info:", user);
    if (user) {
      // update user
      user.username = username.toLowerCase();
      user.name = name;
      user.avatar_url = avatar_url;
      user.bio = bio;
      user.company = company;
      user.location = location;
      user.created_at = created_at;
      user.git_level = git_level;
      user.git_score = git_score;
      user.git_badge = git_badge;
      user.git_points = git_points;
      user.total_prs = total_prs;
      user.total_issues = total_issues;
      user.total_stars = total_stars;
      user.total_commits = total_commits;
      user.contributed_to = contributed_to;
      user.latestUpdate = Date.now();

      await user.save();
      return res.status(202).json({
        msg: "user has been updated successfully!",
      });
    } else {
      console.log("You have taken the wrong route to create the user");
      return res.status(201).json({
        msg: "You have taken the wrong route to create the user",
      });
    }
  } catch (err) {
    return res.status(500).send("Server issue: " + err);
  }
};

// deprecated (only for dev purposes)
const getUserInfo = async (req, res, next) => {
  const url = "https://api.github.com/users/dasunx";
  const response = await axios.get(url);
  const result = await response.data;

  let repo_res;
  let repo_data;

  let user;
  if (result) {
    user = {
      username: result.login,
      name: result.name,
      avatar_url: result.avatar_url || null,
      location: result.location,
      blog: result.blog || null,
      bio: result.bio || null,
    };
    if (result.repos_url) {
      repo_res = await axios.get(result.repos_url);
      repo_data = await repo_res.data;
      // total_stargazer_count & repo count
      let total_star_received = 0;
      let total_public_repos = 0;
      repo_data.map((r) => {
        total_star_received += r.stargazers_count;
        total_public_repos++;
      });
      userWithRepo = { ...user, total_star_received, total_public_repos };
    }
    // create user obj(mongo)

    const newUser = new User(user);
    try {
      await newUser.save(); //saving in the db
    } catch (err) {
      const error = new HttpError(
        "Data hasn't been saved, reason: " + err,
        500
      );
      return next(error);
    }

    res.status(200).json({
      userInfo: userWithRepo,
      msg: "success",
      repos: repo_data,
    });
  } else {
    res.status(403).json({
      msg: "something went wrong",
    });
  }
};

exports.getUserInfo = getUserInfo;
exports.saveUserGitInfo = saveUserGitInfo;
exports.checkUserAvailabity = checkUserAvailabity;
exports.createUser = createUser;
exports.getAllUserNames = getAllUserNames;
exports.sortedLeaderboardUsers = sortedLeaderboardUsers;
exports.getUserById = getUserById;
exports.checkGitAccountAvailability = checkGitAccountAvailability;
